package org.foxarmy.barcodescannerforemployees

import okhttp3.*
import okhttp3.MediaType.Companion.toMediaType
import okhttp3.MediaType.Companion.toMediaTypeOrNull
import okhttp3.RequestBody.Companion.asRequestBody
import org.foxarmy.barcodescannerforemployees.dataclasses.AbstractProduct
import org.foxarmy.barcodescannerforemployees.dataclasses.Category
import org.foxarmy.barcodescannerforemployees.dataclasses.Product
import java.io.File
import java.io.FileOutputStream
import java.io.IOException
import kotlin.concurrent.thread

class Net {
    var language = "en-US"
    var server = "bsfe.foxarmy.org"
    var token = ""

    fun requestProductFromOnlineDB(barcode: String): String {
        var response = ""
        thread {
            val url = "https://ean-online.ru/match.php"

            val client = OkHttpClient()

            val body = FormBody.Builder()
                .add("barcode", barcode)
                .build()

            val request = Request.Builder()
                .url(url)
                .post(body)
                .addHeader("referer", "https://ean-online.ru")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute().body!!.string()
        }.join()

        return if (response == "") {
            "Not found 404"
        } else {
            response
        }
    }

    fun registerAccount(username: String, password: String): Response {
        lateinit var response: Response
        thread {
            val client = OkHttpClient()

            val body = FormBody.Builder()
                .add("username", username)
                .add("password", password)
                .build()

            val request = Request.Builder()
                .url("https://$server/api/user/register")
                .post(body)
                .addHeader("content-type", "application/x-www-form-urlencoded")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun login(username: String, password: String): Response {
        lateinit var response: Response
        thread {
            val client = OkHttpClient()

            val body = FormBody.Builder()
                .add("username", username)
                .add("password", password)
                .build()
            val requestLogin = Request.Builder()
                .url("https://$server/api/user/login")
                .post(body)
                .addHeader("content-type", "application/x-www-form-urlencoded")
                .addHeader("accept-language", language)
                .build()
            response = client.newCall(requestLogin).execute()
        }.join()

        return response
    }

    fun uploadAbstractProduct(groupId: Int, abstractProduct: AbstractProduct, imageFile: File): Response {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val body = MultipartBody.Builder()
                .setType("multipart/form-data".toMediaType())
                .addFormDataPart("file", imageFile.name, imageFile.asRequestBody("image/png".toMediaTypeOrNull()))
                .addFormDataPart("groupId", groupId.toString())
                .addFormDataPart("localId", abstractProduct.id.toString())
                .addFormDataPart("barcode", abstractProduct.barcode)
                .addFormDataPart("name", abstractProduct.name)
                .addFormDataPart("net_weight", abstractProduct.netWeight.toString())
                .addFormDataPart("category", abstractProduct.category.toString())
                .addFormDataPart("unit", abstractProduct.unit.toString())
                .build()

            val request = Request.Builder()
                .url("https://$server/api/abstractproduct/create")
                .post(body)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun changeUsername(newUsername: String): Response {

        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val body = FormBody.Builder()
            body.add("username", newUsername)

            val requestBody = body.build()

            val request = Request.Builder()
                .url("https://$server/api/user/changeUsername")
                .post(requestBody)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()
            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun changePassword(newPassword: String): Response {

        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val body = FormBody.Builder()
            body.add("password", newPassword)

            val requestBody = body.build()

            val request = Request.Builder()
                .url("https://$server/api/user/changePassword")
                .post(requestBody)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()
            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun createGroup(name: String, password: String): Response {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val body = FormBody.Builder()

            val requestBody = body.build()

            val request = Request.Builder()
                .url("https://$server/api/group/create/$name")
                .post(requestBody)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()
            response = client.newCall(request).execute()
        }.join()


        changeGroupPassword(name, password)

        return response
    }

    fun joinGroup(id: Int, password: String): Response {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val body = FormBody.Builder()
                .add("password", password)
                .build()

            val request = Request.Builder()
                .url("https://$server/api/group/join/$id")
                .post(body)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()
            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun changeGroupPassword(name: String, password: String): Response {
        lateinit var response: Response

        thread {

            val groupId = getGroupId(name);

            val client = OkHttpClient()

            val body = FormBody.Builder()
            body.add("password", password)

            val requestBody = body.build()

            val request = Request.Builder()
                .url("https://$server/api/group/password/$groupId")
                .post(requestBody)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()
            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun getGroupId(name: String): String {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/group/byName/$name")
                .get()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()
            response = client.newCall(request).execute()
        }.join()

        val responseText = response.body!!.string()

        return responseText
    }

    fun getGroupName(id: Int): String {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/group/byId/$id")
                .get()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()
            response = client.newCall(request).execute()
        }.join()

        val responseText = response.body!!.string()

        return responseText
    }

    fun getUsersInGroup(groupId: Int): Response {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/group/getUsers/$groupId")
                .get()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun getMyGroups(): Response {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/user/myGroups")
                .get()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun getUsernameById(userId: Int): Response {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/user/byId/$userId")
                .get()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun getGroupAdminId(groupId: Int): Response {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/group/adminId/$groupId")
                .get()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun updateAbstractProduct(groupId: Int, abstractProduct: AbstractProduct, imageFile: File): Response {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val body = MultipartBody.Builder()
                .setType("multipart/form-data".toMediaType())
                .addFormDataPart("file", imageFile.name, imageFile.asRequestBody("image/png".toMediaTypeOrNull()))
                .addFormDataPart("groupId", groupId.toString())
                .addFormDataPart("localId", abstractProduct.id.toString())
                .addFormDataPart("barcode", abstractProduct.barcode)
                .addFormDataPart("name", abstractProduct.name)
                .addFormDataPart("net_weight", abstractProduct.netWeight.toString())
                .addFormDataPart("category", abstractProduct.category.toString())
                .addFormDataPart("unit", abstractProduct.unit.toString())
                .build()

            val request = Request.Builder()
                .url("https://$server/api/abstractproduct/update")
                .post(body)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun uploadCategory(groupId: Int, category: Category): Response {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val body = FormBody.Builder()
                .add("localId", category.id.toString())
                .add("categoryName", category.name)
                .add("groupId", groupId.toString())
                .build()

            val request = Request.Builder()
                .url("https://$server/api/category/create")
                .post(body)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun updateCategory(groupId: Int, category: Category): Response {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val body = FormBody.Builder()
                .add("localId", category.id.toString())
                .add("categoryName", category.name)
                .add("groupId", groupId.toString())
                .build()

            val request = Request.Builder()
                .url("https://$server/api/category/update")
                .post(body)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun uploadProduct(groupId: Int, product: Product): Response {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val body = FormBody.Builder()
                .add("localId", product.id.toString())
                .add("groupId", groupId.toString())
                .add("abstract_product_id", product.abstractProductId.toString())
                .add("amount", product.amount.toString())
                .add("date_of_production", product.dateOfProduction.toString())
                .add("expiry_date", product.dateOfExpiry.toString())
                .build()

            val request = Request.Builder()
                .url("https://$server/api/product/create")
                .post(body)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun updateProduct(groupId: Int, product: Product): Response {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val body = FormBody.Builder()
                .add("localId", product.id.toString())
                .add("groupId", groupId.toString())
                .add("abstract_product_id", product.abstractProductId.toString())
                .add("amount", product.amount.toString())
                .add("date_of_production", product.dateOfProduction.toString())
                .add("expiry_date", product.dateOfExpiry.toString())
                .build()

            val request = Request.Builder()
                .url("https://$server/api/product/update")
                .post(body)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun synchronize(groupId: Int): Response {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/user/synchronize/$groupId")
                .get()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun getProduct(groupId: Int, localId: Int): Response {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/product/$groupId/$localId")
                .get()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun getAbstractProduct(groupId: Int, localId: Int): Response {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/abstractproduct/$groupId/$localId")
                .get()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun downloadImage(url: String, file: File) {
        thread {
            val client = OkHttpClient()
            val request = Request.Builder()
                .url(url)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            client.newCall(request).execute().use { response ->
                if (!response.isSuccessful) throw IOException("Unexpected code $response")

                val fos = FileOutputStream(file)

                response.body?.byteStream()?.use { inputStream ->
                    fos.use {
                        inputStream.copyTo(fos)
                    }
                }
            }
        }.join()
    }

    fun deleteCategory(groupId: Int, localId: Int): Response {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/category/$groupId/$localId")
                .delete()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun deleteAbstractProduct(groupId: Int, localId: Int): Response {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/abstractproduct/$groupId/$localId")
                .delete()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
        }.join()

        return response
    }

    fun deleteProduct(groupId: Int, localId: Int): Response {
        lateinit var response: Response

        thread {
            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/product/$groupId/$localId")
                .delete()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
        }.join()

        return response
    }
}