package org.foxarmy.barcodescannerforemployees.activities

import android.content.DialogInterface
import android.content.Intent
import android.content.SharedPreferences
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.widget.*
import androidx.activity.result.contract.ActivityResultContracts
import androidx.annotation.RequiresApi
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.core.widget.addTextChangedListener
import androidx.security.crypto.EncryptedSharedPreferences
import androidx.security.crypto.MasterKeys
import com.journeyapps.barcodescanner.ScanContract
import com.journeyapps.barcodescanner.ScanIntentResult
import com.journeyapps.barcodescanner.ScanOptions
import okhttp3.Response
import org.foxarmy.barcodescannerforemployees.*
import org.foxarmy.barcodescannerforemployees.database.AbstractProductDAO
import org.foxarmy.barcodescannerforemployees.database.CategoryDAO
import org.foxarmy.barcodescannerforemployees.database.DBStorageController
import org.foxarmy.barcodescannerforemployees.dataclasses.AbstractProduct
import java.io.File
import java.nio.file.Files
import java.nio.file.StandardCopyOption
import kotlin.concurrent.thread

class AddAbstractProductActivity : AppCompatActivity() {
    private lateinit var imageView: ImageView

    private lateinit var saveButton: Button
    private lateinit var takePictureButton: Button
    private lateinit var scanButton: Button

    private lateinit var barcodeText: EditText
    private lateinit var noBarcodeCheckBox: CheckBox
    private lateinit var productNameText: TextView
    private lateinit var netWeightText: TextView
    private lateinit var unitTypeSpinner: Spinner

    private lateinit var categorySpinner: Spinner

    private var abstractProduct: AbstractProduct? = null
    private lateinit var pictureFile: File
    private lateinit var picturesPath: File
    private var barcode: String = ""
    private var action: String = "new"

    private var scanningBarcode = false

    private lateinit var DAO: AbstractProductDAO
    private lateinit var sharedPreferences: SharedPreferences

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setContentView(R.layout.fragment_add_abstract_product)

        sharedPreferences = EncryptedSharedPreferences.create(
            "sensitive",
            MasterKeys.getOrCreate(MasterKeys.AES256_GCM_SPEC),
            applicationContext,
            EncryptedSharedPreferences.PrefKeyEncryptionScheme.AES256_SIV,
            EncryptedSharedPreferences.PrefValueEncryptionScheme.AES256_GCM
        )

        val dbHelper = DBStorageController(this, sharedPreferences.getString("currentGroup", "database")!!)
        DAO = AbstractProductDAO(dbHelper)

        picturesPath = File(filesDir, "pictures")
        val thumbnailsDir = File(cacheDir, "thumbnails")
        thumbnailsDir.mkdirs()
        picturesPath.mkdirs()
        imageView = findViewById(R.id.imageView)

        saveButton = findViewById(R.id.saveButton)
        takePictureButton = findViewById(R.id.takePictureButton)
        scanButton = findViewById(R.id.scan_button)

        barcodeText = findViewById(R.id.barcodeTextEdit)
        noBarcodeCheckBox = findViewById(R.id.noBarcodeCheckBox)
        productNameText = findViewById(R.id.productName)
        netWeightText = findViewById(R.id.netWeight)
        unitTypeSpinner = findViewById(R.id.unitTypeSpinner)

        categorySpinner = findViewById(R.id.categorySpinner)

        fillupCategorySpinner()
        fillupUnitsSpinner()

        noBarcodeCheckBox.setOnClickListener {
            if (noBarcodeCheckBox.isChecked) {
                barcodeText.setText("")
            }
        }

        barcodeText.addTextChangedListener {
            this.barcode = barcodeText.text.toString()
        }

        val extras = intent.extras
        action = extras!!.get("action") as String
        when (action) {
            "update" -> {
                abstractProduct = extras.get("abstractProduct") as AbstractProduct?
            }

            "new_from_barcode" -> {
                abstractProduct = extras.get("abstractProduct") as AbstractProduct?
                barcode = abstractProduct!!.barcode
                performRequest(abstractProduct!!.barcode)
            }
        }

        if (abstractProduct != null) {
            val imageThumbnailUri = getImageUri(this, File(thumbnailsDir, "${abstractProduct!!.imageHash}.webp"))
            pictureFile = File(picturesPath, "${abstractProduct!!.imageHash}.png]")
            imageView.setImageURI(imageThumbnailUri)
            barcodeText.setText(abstractProduct!!.barcode)
            productNameText.text = abstractProduct!!.name
            netWeightText.text = abstractProduct!!.netWeight.toString()
            categorySpinner.setSelection(abstractProduct!!.category)
            unitTypeSpinner.setSelection(abstractProduct!!.unit)
        }

        saveButton.setOnClickListener {
            val productName = productNameText.text.toString()
            val netWeight = netWeightText.text
            if (action != "update" && (!this::pictureFile.isInitialized || !pictureFile.exists())) {
                Toast.makeText(this, getString(R.string.product_picture_request), Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }

            if (barcode == "" && !noBarcodeCheckBox.isChecked) {
                Toast.makeText(this, getString(R.string.product_barcode_request), Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }

            if (productName == "") {
                Toast.makeText(this, getString(R.string.product_name_request), Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }
            if (netWeight.toString() == "" || netWeight.toString().toDoubleOrNull() == null) {
                Toast.makeText(this, getString(R.string.product_net_weight_request), Toast.LENGTH_SHORT).show()
            }

            val net = Net()

            net.token = sharedPreferences.getString("token", "")!!
            net.server = sharedPreferences.getString("server", "")!!
            net.language = sharedPreferences.getString("language", "en-US")!!

            val currentGroup = sharedPreferences.getString("currentGroup", "offline")!!

            lateinit var response: Response

            abstractProduct = AbstractProduct(
                if(abstractProduct == null) 0 else abstractProduct!!.id,
                barcode,
                productName,
                netWeight.toString().toDouble(),
                pictureFile.nameWithoutExtension,
                categorySpinner.selectedItemPosition + 1,
                unitTypeSpinner.selectedItemPosition
            )
            val pictureFile = File(File(filesDir, "pictures"), "${abstractProduct!!.imageHash}.png")

            if (action == "update") {
                DAO.updateAbstractProduct(abstractProduct!!)
                response = net.updateAbstractProduct(currentGroup.toInt(), abstractProduct!!, pictureFile)
            } else if (action == "new" || action == "new_from_barcode") {
                abstractProduct!!.id = DAO.addAbstractProduct(abstractProduct!!).toInt()
                response = net.uploadAbstractProduct(currentGroup.toInt(), abstractProduct!!, pictureFile);
            }
            Toast.makeText(this, response.body!!.string(), Toast.LENGTH_LONG).show()

            finish()
        }

        takePictureButton.setOnClickListener {
            requestPermissionLauncher.launch(android.Manifest.permission.CAMERA)
        }

        scanButton.setOnClickListener {
            scanningBarcode = true
            requestPermissionLauncher.launch(android.Manifest.permission.CAMERA)
        }
    }

    fun performRequest(barcode: String) {
        barcodeText.setText(this.barcode)
        val net = Net();
        val result = net.requestProductFromOnlineDB(barcode)
        Log.d("QWERTYUIOP", "Result of request: $result")

        var abstractProduct: AbstractProduct

        if (DAO.findAbstractProductByBarcode(this.barcode) != null) {
            AlertDialog.Builder(this)
                .setMessage(getString(R.string.abstract_product_already_exists))
                .setPositiveButton(getString(R.string.quit)) { _: DialogInterface, _: Int ->
                    finish()
                }
                .setNegativeButton(getString(R.string.edit_existing)) { _: DialogInterface, _: Int ->
                    val addProductIntent = Intent(this, AddAbstractProductActivity::class.java)
                    val extras = Bundle()
                    val existingAbstractProduct = DAO.findAbstractProductByBarcode(this.barcode)

                    extras.putParcelable("abstractProduct", existingAbstractProduct)
                    addProductIntent.putExtras(extras)
                    ContextCompat.startActivity(this, addProductIntent, extras)
                    finish()
                }.show()
        }

        thread {
            if (result == "Not found 404") {
                runOnUiThread {
                    Toast.makeText(this, getString(R.string.no_product_in_online_database), Toast.LENGTH_LONG)
                        .show()
                    productNameText.setText("")
                    netWeightText.setText("")
                }
                return@thread
            }

            abstractProduct = Parser().parse(result)
            runOnUiThread {
                productNameText.text = abstractProduct.name
                netWeightText.text = abstractProduct.netWeight.toString()
                unitTypeSpinner.setSelection(abstractProduct.unit)
            }
        }
    }

    private fun fillupUnitsSpinner() {
        val units = listOf(
            getString(R.string.kilogram),
            getString(R.string.gram),
            getString(R.string.liter),
            getString(R.string.milliliter),
            getString(R.string.pieces)
        )

        val arrayAdapter =
            ArrayAdapter(this, androidx.appcompat.R.layout.support_simple_spinner_dropdown_item, units)
        arrayAdapter.setDropDownViewResource(androidx.appcompat.R.layout.support_simple_spinner_dropdown_item)
        unitTypeSpinner.adapter = arrayAdapter
    }

    fun fillupCategorySpinner() {

        val categoriesDAO = CategoryDAO(DBStorageController(this, sharedPreferences.getString("currentGroup", "database")!!))

        val categories = categoriesDAO.getAllCategories().map { category -> category.name }

        val arrayAdapter =
            ArrayAdapter(this, androidx.appcompat.R.layout.support_simple_spinner_dropdown_item, categories)
        arrayAdapter.setDropDownViewResource(androidx.appcompat.R.layout.support_simple_spinner_dropdown_item)
        categorySpinner.adapter = arrayAdapter
    }

    @RequiresApi(Build.VERSION_CODES.R)
    val takePicture = registerForActivityResult(ActivityResultContracts.TakePicture()) { success: Boolean ->
        if (success) {
            //Move picture to a proper directory according to its calculated hash
            val tempfile = File(filesDir, "image.png")
            val imageHash = calculateMd5Hash(tempfile)
//            val imageContent = tempfile.inputStream().readBytes()
//            val imageHash = imageContent.toString(Charsets.UTF_8).md5()

            pictureFile = File(picturesPath, "$imageHash.png")
            Files.move(tempfile.toPath(), pictureFile.toPath(), StandardCopyOption.REPLACE_EXISTING)
            tempfile.delete()
            generateThumbnailForImage(this, imageHash)

            imageView.setImageURI(getImageUri(this, pictureFile))
        } else {
            Log.e("QWERTYUIOP", "Cannot save a picture")
        }
    }

    @RequiresApi(Build.VERSION_CODES.R)
    fun getPicture() {
        //Saving picture to a temp file for further hash calculation and moving to a proper directory
        val imageFile = File(this.filesDir, "image.png")
        val imageUri = getImageUri(this, imageFile)
        if (imageUri != null) {
            takePicture.launch(imageUri)
        }
    }

    @RequiresApi(Build.VERSION_CODES.R)
    val requestPermissionLauncher =
        registerForActivityResult(
            ActivityResultContracts.RequestPermission()
        ) { isGranted: Boolean ->
            if (isGranted) {
                if (scanningBarcode) {
                    prepareBarcodeScanner()
                } else {
                    getPicture()
                }
            } else {
                Toast.makeText(this, getString(R.string.camera_permission_for_picture_request), Toast.LENGTH_LONG)
                    .show()
            }
        }

    private val scanLauncher = registerForActivityResult(ScanContract()) { result: ScanIntentResult ->
        run {
            if (result.contents == null) {
                Toast.makeText(this, getString(R.string.cancelled), Toast.LENGTH_SHORT).show()
            } else {
                val scannedBarcode = result.contents
                barcodeText.setText(scannedBarcode)
                performRequest(scannedBarcode)
            }
            scanningBarcode = false
        }
    }

    private fun prepareBarcodeScanner() {
        val options = ScanOptions()
        options.setDesiredBarcodeFormats(ScanOptions.EAN_13)
        options.setPrompt(getString(R.string.scan_barcode_of_a_product))
        options.setCameraId(0)
        options.setBeepEnabled(false)
        options.setBarcodeImageEnabled(true)
        options.setOrientationLocked(false)

        scanLauncher.launch(options)
    }
}