package org.foxarmy.barcodescannerforemployees.activities

import android.app.Activity
import android.content.SharedPreferences
import android.os.Bundle
import android.widget.Button
import android.widget.EditText
import android.widget.Toast
import androidx.security.crypto.EncryptedSharedPreferences
import androidx.security.crypto.MasterKeys
import org.foxarmy.barcodescannerforemployees.Net
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.database.CategoryDAO
import org.foxarmy.barcodescannerforemployees.database.DBStorageController
import org.foxarmy.barcodescannerforemployees.dataclasses.Category

class AddCategoryActivity : Activity() {

    private lateinit var DAO: CategoryDAO
    private lateinit var sharedPreferences: SharedPreferences

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setContentView(R.layout.activity_add_category)

        sharedPreferences = EncryptedSharedPreferences.create(
            "sensitive",
            MasterKeys.getOrCreate(MasterKeys.AES256_GCM_SPEC),
            applicationContext,
            EncryptedSharedPreferences.PrefKeyEncryptionScheme.AES256_SIV,
            EncryptedSharedPreferences.PrefValueEncryptionScheme.AES256_GCM
        )

        val dbHelper = DBStorageController(this, sharedPreferences.getString("currentGroup", "offline")!!)
        DAO = CategoryDAO(dbHelper)

        val extras = intent.extras
        val category = extras!!.get("category") as Category?

        val categoryNameTextEdit: EditText = findViewById(R.id.newCategoryName)

        categoryNameTextEdit.setText(category!!.name)

        val net = Net()

        net.token = sharedPreferences.getString("token", "")!!
        net.server = sharedPreferences.getString("server", "")!!
        net.language = sharedPreferences.getString("language", "")!!

        val currentGroup = sharedPreferences.getString("currentGroup", "offline")!!.toInt()

        findViewById<Button>(R.id.saveButton).setOnClickListener {
            if (categoryNameTextEdit.text.toString() == "") {
                Toast.makeText(this, getString(R.string.category_name_required), Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }

            if (category.id == 0) { // Inserting new category
                val newCategory = Category(0, categoryNameTextEdit.text.toString())
                newCategory.id = DAO.addCategory(newCategory).toInt()
                net.uploadCategory(currentGroup, newCategory)
            } else { // Updating existing category
                category.name = categoryNameTextEdit.text.toString()
                DAO.updateCategory(category)
                net.updateCategory(currentGroup, category)
            }

            finish()
        }
    }
}