package org.foxarmy.barcodescannerforemployees.activities

import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import android.widget.LinearLayout
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.security.crypto.EncryptedSharedPreferences
import androidx.security.crypto.MasterKeys
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.database.DBStorageController
import org.foxarmy.barcodescannerforemployees.database.ProductDAO
import org.foxarmy.barcodescannerforemployees.databinding.ActivityExpiryCalendarBinding
import org.foxarmy.barcodescannerforemployees.views.ExpiryGroupView

class ExpiryCalendarActivity : AppCompatActivity() {
    private lateinit var binding: ActivityExpiryCalendarBinding

    private lateinit var productDAO: ProductDAO
    private lateinit var sharedPreferences: SharedPreferences

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setContentView(R.layout.fragment_expiry_dates)

        sharedPreferences = EncryptedSharedPreferences.create(
            "sensitive",
            MasterKeys.getOrCreate(MasterKeys.AES256_GCM_SPEC),
            applicationContext,
            EncryptedSharedPreferences.PrefKeyEncryptionScheme.AES256_SIV,
            EncryptedSharedPreferences.PrefValueEncryptionScheme.AES256_GCM
        )

        val dbHelper = DBStorageController(this, sharedPreferences.getString("currentGroup", "database")!!)
        productDAO = ProductDAO(dbHelper)

        fillUp()
    }

    fun displayDate(date:Long) {
        val expiryCalendarGroupActivityIntent = Intent(this, ExpiryCalendarGroupActivity::class.java)
        val extras = Bundle()
        extras.putLong("date", date)
        expiryCalendarGroupActivityIntent.putExtras(extras)
        ContextCompat.startActivity(this, expiryCalendarGroupActivityIntent, extras)
    }

    private fun fillUp() {
        val dates = productDAO.findAllExpiryDates()

        val container = findViewById<LinearLayout>(R.id.datesLinearLayout)

        dates.forEach { date ->
            val newDate = ExpiryGroupView(this, this as Context, date)
            container.addView(newDate)
        }
    }
}