package org.foxarmy.barcodescannerforemployees.activities

import android.content.Intent
import android.os.Bundle
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.security.crypto.EncryptedSharedPreferences
import androidx.security.crypto.MasterKeys
import org.foxarmy.barcodescannerforemployees.Net
import org.foxarmy.barcodescannerforemployees.databinding.ActivityGroupBinding

class GroupActivity : AppCompatActivity() {

    private lateinit var binding: ActivityGroupBinding

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        binding = ActivityGroupBinding.inflate(layoutInflater)
        setContentView(binding.root)

        val sharedPreferences = EncryptedSharedPreferences.create(
            "sensitive",
            MasterKeys.getOrCreate(MasterKeys.AES256_GCM_SPEC),
            applicationContext,
            EncryptedSharedPreferences.PrefKeyEncryptionScheme.AES256_SIV,
            EncryptedSharedPreferences.PrefValueEncryptionScheme.AES256_GCM
        )

        binding.createGroupButton.setOnClickListener {
            val groupName = binding.groupNameTextEdit.text.toString()
            val groupPassword = binding.groupPasswordTextEdit.text.toString()
            // group is set to "successful"
            val n = Net()
            n.language = sharedPreferences.getString("language", "en-US")!!
            n.server = sharedPreferences.getString("server", "")!!
            n.token = sharedPreferences.getString("token", "")!!

            val response = n.createGroup(groupName, groupPassword)
            val responseText = response.body!!.string()

            if (response.code == 200) {
                val currentGroups = sharedPreferences.getStringSet("groups", mutableSetOf())
                currentGroups!!.add(responseText)
                sharedPreferences.edit().putStringSet("groups", currentGroups).apply()
                sharedPreferences.edit().putString("currentGroup", responseText).apply()
                val intent = Intent(this, MainActivity::class.java)
                startActivity(intent)
                finish()
            } else {
                Toast.makeText(this, responseText, Toast.LENGTH_LONG).show()
            }
        }

        binding.joinGroupButton.setOnClickListener {
            val groupName = binding.groupNameTextEdit.text.toString()
            val groupPassword = binding.groupPasswordTextEdit.text.toString()

            val n = Net()
            n.language = sharedPreferences.getString("language", "en-US")!!
            n.server = sharedPreferences.getString("server", "")!!
            n.token = sharedPreferences.getString("token", "")!!

            val groupId = n.getGroupId(groupName).toInt()
            val response = n.joinGroup(groupId, groupPassword)
            val responseText = response.body!!.string()

            if (response.code == 200) {
                val currentGroups = sharedPreferences.getStringSet("groups", mutableSetOf())
                currentGroups!!.add(groupId.toString())
                sharedPreferences.edit().putStringSet("groups", currentGroups).apply()
                sharedPreferences.edit().putString("currentGroup", groupId.toString()).apply()
                val intent = Intent(this, MainActivity::class.java)
                startActivity(intent)
                finish()
            } else {
                Toast.makeText(this, responseText, Toast.LENGTH_LONG).show()
            }
        }
    }
}