package org.foxarmy.barcodescannerforemployees.activities

import android.content.DialogInterface
import android.content.Intent
import android.os.Bundle
import android.view.Menu
import android.view.MenuItem
import androidx.appcompat.app.ActionBarDrawerToggle
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.drawerlayout.widget.DrawerLayout
import androidx.security.crypto.EncryptedSharedPreferences
import androidx.security.crypto.MasterKeys
import androidx.viewpager.widget.ViewPager
import com.google.android.material.navigation.NavigationView
import org.foxarmy.barcodescannerforemployees.Net
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.ViewPagerAdapter
import org.foxarmy.barcodescannerforemployees.convertToUnixEpochTimestamp
import org.foxarmy.barcodescannerforemployees.database.AbstractProductDAO
import org.foxarmy.barcodescannerforemployees.database.CategoryDAO
import org.foxarmy.barcodescannerforemployees.database.DBStorageController
import org.foxarmy.barcodescannerforemployees.database.ProductDAO
import org.foxarmy.barcodescannerforemployees.databinding.ActivityMainBinding
import org.foxarmy.barcodescannerforemployees.dataclasses.AbstractProduct
import org.foxarmy.barcodescannerforemployees.dataclasses.Category
import org.foxarmy.barcodescannerforemployees.dataclasses.Product
import org.foxarmy.barcodescannerforemployees.fragments.CategoriesFragment
import org.foxarmy.barcodescannerforemployees.fragments.ShelfFragment
import org.foxarmy.barcodescannerforemployees.fragments.StorageFragment
import org.json.JSONArray
import org.json.JSONObject
import java.io.File


class MainActivity : AppCompatActivity(), NavigationView.OnNavigationItemSelectedListener {

    private lateinit var binding: ActivityMainBinding
    lateinit var adapter: ViewPagerAdapter
    var drawerLayout: DrawerLayout? = null
    var actionBarDrawerToggle: ActionBarDrawerToggle? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)

        setSupportActionBar(binding.toolbar)
        setupViewPager(binding.tabViewpager)
        binding.tabTablayout.setupWithViewPager(binding.tabViewpager)

        drawerLayout = binding.drawerLayout
        actionBarDrawerToggle = ActionBarDrawerToggle(this, drawerLayout, R.string.nav_open, R.string.nav_close)

        drawerLayout!!.addDrawerListener(actionBarDrawerToggle!!)
        actionBarDrawerToggle!!.syncState()
        binding.navView.setNavigationItemSelectedListener(this)

        // to make the Navigation drawer icon always appear on the action bar
        supportActionBar!!.setDisplayHomeAsUpEnabled(true)

        binding.expiryCalendarFab.setOnClickListener { _ ->
            val expiryCalendarIntent = Intent(this, ExpiryCalendarActivity::class.java)
            val extras = Bundle()
            expiryCalendarIntent.putExtras(extras)
            ContextCompat.startActivity(this, expiryCalendarIntent, extras)
        }

        binding.newElementFab.setOnClickListener { view ->
            val currentPosition = binding.tabTablayout.selectedTabPosition
            val fragment = adapter.getItem(currentPosition)

            when (fragment::class.simpleName.toString()) {
                "StorageFragment" -> {
                    val addAbstractProductIntent = Intent(this, AddAbstractProductActivity::class.java)
                    val extras = Bundle()
                    // I reuse the same stuff for editing and adding new product.
                    // if abstractProduct == null, it means that we need to create new object
                    extras.putParcelable("abstractProduct", null)
                    extras.putString("action", "new")
                    addAbstractProductIntent.putExtras(extras)
                    ContextCompat.startActivity(this, addAbstractProductIntent, extras)
                }

                "CategoriesFragment" -> {
                    val addCategoryIntent = Intent(this, AddCategoryActivity::class.java)
                    val extras = Bundle()
                    extras.putParcelable("category", Category(0, ""))
                    addCategoryIntent.putExtras(extras)
                    ContextCompat.startActivity(this, addCategoryIntent, extras)
                }

                "ShelfFragment" -> {
                    val addProductIntent = Intent(this, AddProductActivity::class.java)
                    val extras = Bundle()
                    extras.putParcelable("product", null)
                    addProductIntent.putExtras(extras)
                    ContextCompat.startActivity(this, addProductIntent, extras)

                }
            }
        }
        synchronize()
    }

    private fun synchronize() {

        val sharedPreferences = EncryptedSharedPreferences.create(
            "sensitive",
            MasterKeys.getOrCreate(MasterKeys.AES256_GCM_SPEC),
            applicationContext,
            EncryptedSharedPreferences.PrefKeyEncryptionScheme.AES256_SIV,
            EncryptedSharedPreferences.PrefValueEncryptionScheme.AES256_GCM
        )

        if (sharedPreferences.getString("currentGroup", "offline")!! == "offline") return

        val net = Net()

        net.language = sharedPreferences.getString("language", "en-US")!!
        net.server = sharedPreferences.getString("server", "")!!
        net.token = sharedPreferences.getString("token", "")!!

        val currentGroup = sharedPreferences.getString("currentGroup", "")!!.toInt()

        val data = JSONObject(net.synchronize(currentGroup).body!!.string())

        val remoteAbstractProducts = data["abstract_products"] as JSONArray
        val remoteProducts = data["products"] as JSONArray
        val remoteCategories = data["categories"] as JSONArray

        val dbHelper = DBStorageController(this, sharedPreferences.getString("currentGroup", "")!!)

        val abstractProductDAO = AbstractProductDAO(dbHelper)
        val productDAO = ProductDAO(dbHelper)
        val categoryDAO = CategoryDAO(dbHelper)

        val localAbstractProducts = abstractProductDAO.getSortedListOfAbstractProducts(0, "", arrayOf(""))
        val localProducts = productDAO.getSortedListOfProducts(0, "", "")
        val localCategories = categoryDAO.getAllCategories()

        syncAbstractProducts(net, currentGroup, abstractProductDAO, remoteAbstractProducts, localAbstractProducts)
        syncProducts(productDAO, remoteProducts, localProducts)
        syncCategories(categoryDAO, remoteCategories, localCategories)
    }

    private fun syncCategories(
        categoryDAO: CategoryDAO,
        remoteCategories: JSONArray,
        localCategories: List<Category>
    ) {
        for (localCategory in localCategories) {
            var found = false
            for (i in 0 until remoteCategories.length()) {
                val remoteProduct = remoteCategories.getJSONObject(i)
                if (remoteProduct["local_id"] == localCategory.id) {
                    found = true
                    break
                }
            }
            if (!found) categoryDAO.eraseCategory(localCategory.id, this)
        }
        for (i in 0 until remoteCategories.length()) {
            var categoryInRemoteDB: JSONObject? = null
            var categoryInLocalDB: Category? = null

            val remoteCategory = remoteCategories.getJSONObject(i)

            for (localCategory in localCategories) {
                if (remoteCategory["local_id"] == localCategory.id) {
                    categoryInRemoteDB = remoteCategory
                    categoryInLocalDB = localCategory
                    break
                }
            }

            if (categoryInLocalDB == null && remoteCategory != null) {
                var newCategory: Category

                with (remoteCategory) {
                    newCategory = Category(
                        this["local_id"].toString().toInt(),
                        this["name"].toString()
                    )
                }

                categoryDAO.addCategory(newCategory)
            }

            if (categoryInRemoteDB != null && categoryInLocalDB != null) {
                if (categoryInRemoteDB["hash"] != categoryInLocalDB.calculateHash()) {

                    var updatedData: Category

                    with(categoryInRemoteDB) {
                        updatedData = Category(
                            this["local_id"].toString().toInt(),
                            this["name"].toString()
                        )
                    }

                    categoryDAO.updateCategory(updatedData)
                }
            }
        }
    }

    private fun syncProducts(
        productDAO: ProductDAO,
        remoteProducts: JSONArray,
        localProducts: List<Product>
    ) {
        for (localProduct in localProducts) {
            var found = false
            for (i in 0 until remoteProducts.length()) {
                val remoteProduct = remoteProducts.getJSONObject(i)
                if (remoteProduct["local_id"] == localProduct.id) {
                    found = true
                    break
                }
            }
            if (!found) productDAO.eraseProduct(localProduct.id)
        }
        for (i in 0 until remoteProducts.length()) {
            var productInRemoteDB: JSONObject? = null
            var productInLocalDB: Product? = null

            val remoteProduct = remoteProducts.getJSONObject(i)

            for (localProduct in localProducts) {
                if (remoteProduct["local_id"] == localProduct.id) {
                    productInRemoteDB = remoteProduct
                    productInLocalDB = localProduct
                    break
                }
            }

            if (productInLocalDB == null && remoteProduct != null) {
                var newProduct: Product

                with(remoteProduct) {
                    newProduct = Product(
                        this["local_id"].toString().toInt(),
                        this["abstract_product_id"].toString().toInt(),
                        this["amount"].toString().toInt(),
                        convertToUnixEpochTimestamp(this["date_of_production"].toString()),
                        convertToUnixEpochTimestamp(this["expiry_date"].toString())
                    )
                }

                productDAO.insertNewProduct(newProduct)
            }

            if (productInRemoteDB != null && productInLocalDB != null) {
                if (productInRemoteDB["hash"] != productInLocalDB.calculateHash()) {

                    var updatedData: Product

                    with(productInRemoteDB) {
                        updatedData = Product(
                            this["local_id"].toString().toInt(),
                            this["abstract_product_id"].toString().toInt(),
                            this["amount"].toString().toInt(),
                            convertToUnixEpochTimestamp(this["date_of_production"].toString()),
                            convertToUnixEpochTimestamp(this["expiry_date"].toString())
                        )
                    }

                    productDAO.updateProduct(updatedData)
                }
            }
        }
    }

    private fun syncAbstractProducts(
        net: Net,
        currentGroup: Int,
        abstractProductDAO: AbstractProductDAO,
        remoteAbstractProducts: JSONArray,
        localAbstractProducts: List<AbstractProduct>
    ) {
        for (localAbstractProduct in localAbstractProducts) {
            var found = false
            for (i in 0 until remoteAbstractProducts.length()) {
                val remoteAbstractProduct = remoteAbstractProducts.getJSONObject(i)
                if (remoteAbstractProduct["local_id"] == localAbstractProduct.id) {
                    found = true
                    break
                }
            }
            if (!found) abstractProductDAO.eraseAbstractProduct(localAbstractProduct.id, this)
        }
        for (i in 0 until remoteAbstractProducts.length()) {
            var abstractProductInRemoteDB: JSONObject? = null
            var abstractProductInLocalDB: AbstractProduct? = null

            val remoteAbstractProduct = remoteAbstractProducts.getJSONObject(i)

            for (localAbstractProduct in localAbstractProducts) {
                if (remoteAbstractProduct["local_id"] == localAbstractProduct.id) {
                    abstractProductInRemoteDB = remoteAbstractProduct
                    abstractProductInLocalDB = localAbstractProduct
                    break
                }
            }

            if (abstractProductInLocalDB == null && remoteAbstractProduct != null) {
                val localId = remoteAbstractProduct["local_id"].toString().toInt()

                val picturesDir = File(filesDir, "pictures")
                picturesDir.mkdirs()

                val pictureFile =
                    File(picturesDir, "${remoteAbstractProduct["image_filename"]}.png")
                val url = "https://${net.server}/api/abstractproduct/getImage/${currentGroup}/${localId}"
                net.downloadImage(url, pictureFile)

                var newAbstractProduct: AbstractProduct

                with(remoteAbstractProduct) {
                    newAbstractProduct = AbstractProduct(
                        this["local_id"].toString().toInt(),
                        this["barcode"].toString(),
                        this["name"].toString(),
                        this["net_weight"].toString().toDouble(),
                        this["image_filename"].toString(),
                        this["category"].toString().toInt(),
                        this["unit"].toString().toInt()
                    )
                }

                abstractProductDAO.addAbstractProduct(newAbstractProduct)

            }

            if (abstractProductInRemoteDB != null && abstractProductInLocalDB != null) {
                if (abstractProductInRemoteDB["hash"] != abstractProductInLocalDB.calculateHash()) {
                    val localId = abstractProductInRemoteDB["local_id"].toString().toInt()

                    val pictureFile =
                        File(File(filesDir, "pictures"), "${abstractProductInRemoteDB["image_filename"]}.png")
                    val url = "https://${net.server}/api/abstractproduct/getImage/${currentGroup}/${localId}"
                    net.downloadImage(url, pictureFile)

                    var updatedData: AbstractProduct

                    with(abstractProductInRemoteDB) {
                        updatedData = AbstractProduct(
                            this["local_id"].toString().toInt(),
                            this["barcode"].toString(),
                            this["name"].toString(),
                            this["net_weight"].toString().toDouble(),
                            this["image_filename"].toString(),
                            this["category"].toString().toInt(),
                            this["unit"].toString().toInt()
                        )
                    }


                    abstractProductDAO.updateAbstractProduct(updatedData)
                }
            }
        }
    }

    private fun setupViewPager(viewpager: ViewPager) {
        adapter = ViewPagerAdapter(supportFragmentManager)

        adapter.addFragment(StorageFragment.newInstance("", arrayOf("")), getString(R.string.storage_title))
        adapter.addFragment(ShelfFragment(), getString(R.string.shelf_title))
        adapter.addFragment(CategoriesFragment(), getString(R.string.categories_title))

        //TODO: Settings

        // setting adapter to view pager.
        viewpager.adapter = adapter
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        menuInflater.inflate(R.menu.menu_main, menu)

        return true
    }

    override fun onNavigationItemSelected(item: MenuItem): Boolean {
        lateinit var intent: Intent
        when (item.itemId) {
            R.id.nav_account -> {
                intent = Intent(this, AccountSettingsActivity::class.java)
            }

            R.id.nav_groups -> {
                intent = Intent(this, MyGroupsActivity::class.java)
            }

            R.id.nav_settings -> {
                //TODO: Settings
                return false
            }
        }
        startActivity(intent)

        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (actionBarDrawerToggle!!.onOptionsItemSelected(item)) {
            return true
        } else {

            val currentPosition = binding.tabTablayout.selectedTabPosition
            val fragment = adapter.getItem(currentPosition)

            return when (item.itemId) {
                R.id.action_settings -> {
                    true
                }

                R.id.action_delete -> {

                    when (fragment::class.simpleName.toString()) {
                        "StorageFragment" -> {
                            AlertDialog.Builder(this)
                                .setMessage(getString(R.string.deleting_abstract_product_warning))
                                .setPositiveButton(getString(R.string.yes)) { _: DialogInterface, _: Int ->
                                    val storageFragment = fragment as StorageFragment
                                    storageFragment.removeSelected()
                                }
                                .setNegativeButton(getString(R.string.no)) { _: DialogInterface, _: Int ->

                                }.show()
                        }

                        "CategoriesFragment" -> {
                            AlertDialog.Builder(this)
                                .setMessage(getString(R.string.deleting_category_warning))
                                .setPositiveButton(getString(R.string.yes)) { _: DialogInterface, _: Int ->
                                    val categoriesFragment = fragment as CategoriesFragment
                                    categoriesFragment.removeSelected()
                                }
                                .setNegativeButton(getString(R.string.no)) { _: DialogInterface, _: Int ->

                                }.show()
                        }

                        "ShelfFragment" -> {
                            val shelfFragment = fragment as ShelfFragment
                            shelfFragment.removeSelected()
                        }
                    }
                    true
                }

                R.id.action_update -> {
                    when (fragment::class.simpleName.toString()) {
                        "StorageFragment" -> {
                            val storageFragment = fragment as StorageFragment
                            storageFragment.updateSelected()
                        }

                        "CategoriesFragment" -> {
                            val categoriesFragment = fragment as CategoriesFragment
                            categoriesFragment.updateSelected()
                        }

                        "ShelfFragment" -> {
                            val shelfFragment = fragment as ShelfFragment
                            shelfFragment.updateSelected()
                        }
                    }
                    true
                }

                else -> super.onOptionsItemSelected(item)
            }
        }

    }

    fun filterAbstractProductsByCategory(id: Int) {
        binding.tabViewpager.setCurrentItem(0, true)

        val currentPosition = binding.tabTablayout.selectedTabPosition
        val fragment = adapter.getItem(currentPosition)

        val storageFragment = fragment as StorageFragment
        storageFragment.filterByCategory(id)
    }
}