package org.foxarmy.barcodescannerforemployees.activities

import android.content.Context
import android.os.Bundle
import android.view.View
import androidx.appcompat.app.AppCompatActivity
import androidx.security.crypto.EncryptedSharedPreferences
import androidx.security.crypto.MasterKeys
import org.foxarmy.barcodescannerforemployees.Net
import org.foxarmy.barcodescannerforemployees.databinding.ActivityManageGroupBinding
import org.foxarmy.barcodescannerforemployees.parseArray
import org.foxarmy.barcodescannerforemployees.views.GroupMemberView

class ManageGroupActivity : AppCompatActivity(){
    private lateinit var binding: ActivityManageGroupBinding
    private var isAdmin: Boolean = true

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        binding = ActivityManageGroupBinding.inflate(layoutInflater)
        setContentView(binding.root)

        val groupId = intent.extras!!.getInt("groupId")

        val masterKeyAlias = MasterKeys.getOrCreate(MasterKeys.AES256_GCM_SPEC)

        val sharedPreferences = EncryptedSharedPreferences.create(
            "sensitive",
            masterKeyAlias,
            applicationContext,
            EncryptedSharedPreferences.PrefKeyEncryptionScheme.AES256_SIV,
            EncryptedSharedPreferences.PrefValueEncryptionScheme.AES256_GCM
        )

        val net = Net()

        net.token = sharedPreferences.getString("token", "")!!
        net.server = sharedPreferences.getString("server", "")!!
        net.language = sharedPreferences.getString("language", "en-US")!!


        val users = parseArray(net.getUsersInGroup(groupId).body!!.string())

        for (user in users) {
            val groupMemberView = GroupMemberView(this, this as Context, net.getUsernameById(user).body!!.string(), user)

            binding.groupsContent.addView(groupMemberView)
        }

        isAdmin = amIAnAdminIn(groupId)

        if (!isAdmin) {
            binding.renameButton.visibility = View.GONE
        }
    }

    fun amIAnAdminIn(groupId: Int): Boolean {
        val sharedPreferences = EncryptedSharedPreferences.create(
            "sensitive",
            MasterKeys.getOrCreate(MasterKeys.AES256_GCM_SPEC),
            applicationContext,
            EncryptedSharedPreferences.PrefKeyEncryptionScheme.AES256_SIV,
            EncryptedSharedPreferences.PrefValueEncryptionScheme.AES256_GCM
        )

        val net = Net()

        net.token = sharedPreferences.getString("token", "")!!
        net.server = sharedPreferences.getString("server", "")!!
        net.language = sharedPreferences.getString("language", "en-US")!!

        val result = sharedPreferences.getInt("userId", 0) == net.getGroupAdminId(groupId).body!!.string().toInt()
        return result
    }
}