package org.foxarmy.barcodescannerforemployees.database

import android.content.ContentValues
import android.content.Context
import android.provider.BaseColumns
import org.foxarmy.barcodescannerforemployees.dataclasses.Category

class CategoryDAO(private val dbHelper: DBStorageController) {
    fun eraseCategory(id: Int, context: Context) {

        val abstractProductDAO = AbstractProductDAO(dbHelper)
        val db = dbHelper.writableDatabase

        val productsInCategory = abstractProductDAO.getAllAbstractProductInCategory(id)

        for (product in productsInCategory.iterator()) {
            abstractProductDAO.eraseAbstractProduct(product.id, context)
        }

        db.delete(CategoriesContract.CategoryEntry.TABLE_NAME, BaseColumns._ID + "=" + id, null)
    }

    fun getCategoryNameById(id: Int): String {

        val db = dbHelper.readableDatabase

        var result = ""

        val projection = arrayOf(CategoriesContract.CategoryEntry.CATEGORY_NAME)
        val selection = "${BaseColumns._ID} = ?"
        val selectionArgs = arrayOf(id.toString())

        val cursor = db.query(
            CategoriesContract.CategoryEntry.TABLE_NAME,
            projection,
            selection,
            selectionArgs,
            null,
            null,
            null
        )

        with(cursor) {
            while (moveToNext()) {
                result = getString(getColumnIndexOrThrow(CategoriesContract.CategoryEntry.CATEGORY_NAME))
            }
        }

        return result
    }

    fun getAllCategories(): List<Category> {

        val db = dbHelper.readableDatabase

        val categories = mutableListOf<Category>()

        val projection = arrayOf(
            BaseColumns._ID,
            CategoriesContract.CategoryEntry.CATEGORY_NAME
        )

        val cursor = db.query(
            CategoriesContract.CategoryEntry.TABLE_NAME,
            projection,
            null,
            null,
            null,
            null,
            BaseColumns._ID + " ASC"
        )

        with(cursor) {
            while (moveToNext()) {
                val category = Category(
                    getInt(getColumnIndexOrThrow(BaseColumns._ID)),
                    getString(getColumnIndexOrThrow(CategoriesContract.CategoryEntry.CATEGORY_NAME))
                )
                categories.add(category)
            }
        }

        return categories
    }

    fun addCategory(category: Category): Long {

        val db = dbHelper.writableDatabase

        val values = ContentValues().apply {
            if (category.id > 0) put(BaseColumns._ID, category.id)
            put(CategoriesContract.CategoryEntry.CATEGORY_NAME, category.name)
        }

        val id = db.insert(CategoriesContract.CategoryEntry.TABLE_NAME, null, values)

        return id
    }

    fun updateCategory(category: Category) {

        val db = dbHelper.writableDatabase

        val values = ContentValues().apply {
            put(CategoriesContract.CategoryEntry.CATEGORY_NAME, category.name)
        }
        db.update(
            CategoriesContract.CategoryEntry.TABLE_NAME,
            values,
            "${BaseColumns._ID} = ?",
            arrayOf(category.id.toString())
        )
    }
}