package org.foxarmy.barcodescannerforemployees.database

import android.content.Context
import android.database.sqlite.SQLiteDatabase
import android.database.sqlite.SQLiteOpenHelper
import android.provider.BaseColumns

object AbstractProductContract {
    object AbstractProductEntry : BaseColumns {
        const val TABLE_NAME = "abstract_products"
        const val BARCODE = "barcode"
        const val PRODUCT_NAME = "name"
        const val PRODUCT_NET_WEIGHT = "net_weight"
        const val IMAGE_FILENAME = "image_filename"
        const val CATEGORY = "category"
        const val UNIT = "unit"
    }
}

object CategoriesContract {
    object CategoryEntry : BaseColumns {
        const val TABLE_NAME = "categories"
        const val CATEGORY_NAME = "category_name"
    }
}

object ProductContract {
    object ProductEntry : BaseColumns {
        const val TABLE_NAME = "products"
        const val ABSTRACT_PRODUCT_ID = "abstract_product_id"
        const val AMOUNT = "amount"
        const val DATE_OF_PRODUCTION = "date_of_production"
        const val EXPIRY_DATE = "expiry_date"
    }
}

const val SQL_CREATE_ABSTRACT_PRODUCTS_TABLE =
    "CREATE TABLE ${AbstractProductContract.AbstractProductEntry.TABLE_NAME} (" +
            "${BaseColumns._ID} INTEGER PRIMARY KEY," +
            "${AbstractProductContract.AbstractProductEntry.BARCODE} TEXT," +
            "${AbstractProductContract.AbstractProductEntry.PRODUCT_NAME} TEXT," +
            "${AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT} REAL," +
            "${AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME} TEXT," +
            "${AbstractProductContract.AbstractProductEntry.CATEGORY} INTEGER," +
            "${AbstractProductContract.AbstractProductEntry.UNIT} INTEGER)"

const val SQL_CREATE_PRODUCTS_TABLE =
    "CREATE TABLE ${ProductContract.ProductEntry.TABLE_NAME} (" +
            "${BaseColumns._ID} INTEGER PRIMARY KEY," +
            "${ProductContract.ProductEntry.ABSTRACT_PRODUCT_ID} INTEGER," +
            "${ProductContract.ProductEntry.AMOUNT} INTEGER," +
            "${ProductContract.ProductEntry.DATE_OF_PRODUCTION} INTEGER," +
            "${ProductContract.ProductEntry.EXPIRY_DATE} INTGER)"

const val SQL_CREATE_CATEGORIES_TABLE =
    "CREATE TABLE ${CategoriesContract.CategoryEntry.TABLE_NAME} (" +
            "${BaseColumns._ID} INTEGER PRIMARY KEY," +
            "${CategoriesContract.CategoryEntry.CATEGORY_NAME} TEXT)"

class DBStorageController(context: Context, dbname: String) :
    SQLiteOpenHelper(context, dbname, null, DATABASE_VERSION) {

        init {
            DATABASE_NAME = "$dbname.db"
        }

    override fun onCreate(db: SQLiteDatabase) {

        db.execSQL(SQL_CREATE_ABSTRACT_PRODUCTS_TABLE)
        db.execSQL(SQL_CREATE_PRODUCTS_TABLE)
        db.execSQL(SQL_CREATE_CATEGORIES_TABLE)
    }

    override fun onUpgrade(db: SQLiteDatabase?, oldV: Int, newV: Int) {
        TODO("Not yet implemented")
    }

    companion object {
        const val DATABASE_VERSION = 1
        var DATABASE_NAME = "database.db"
    }
}