package org.foxarmy.barcodescannerforemployees.fragments

import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.Toast
import androidx.core.content.ContextCompat
import androidx.core.view.children
import androidx.fragment.app.Fragment
import androidx.security.crypto.EncryptedSharedPreferences
import androidx.security.crypto.MasterKeys
import org.foxarmy.barcodescannerforemployees.Net
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.activities.AddCategoryActivity
import org.foxarmy.barcodescannerforemployees.database.CategoryDAO
import org.foxarmy.barcodescannerforemployees.database.DBStorageController
import org.foxarmy.barcodescannerforemployees.views.CategoryView

class CategoriesFragment : Fragment() {

    private lateinit var sharedPreferences: SharedPreferences
    private lateinit var categoryDAO: CategoryDAO

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        sharedPreferences = EncryptedSharedPreferences.create(
            "sensitive",
            MasterKeys.getOrCreate(MasterKeys.AES256_GCM_SPEC),
            requireContext(),
            EncryptedSharedPreferences.PrefKeyEncryptionScheme.AES256_SIV,
            EncryptedSharedPreferences.PrefValueEncryptionScheme.AES256_GCM
        )

        val dbHelper = DBStorageController(requireContext(), sharedPreferences.getString("currentGroup", "database")!!)
        categoryDAO = CategoryDAO(dbHelper)
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        return inflater.inflate(R.layout.fragment_categories, container, false)
    }

    override fun onResume() {
        super.onResume()

        updateContent()
    }

    fun removeSelected() {
        val layout = view?.findViewById<LinearLayout>(R.id.categoriesLayout)

        var deleted = false
        for (view: CategoryView in layout?.children!!.iterator() as Iterator<CategoryView>) {
            if (view.isCategorySelected) {
                val net = Net()
                net.token = sharedPreferences.getString("token", "")!!
                net.server = sharedPreferences.getString("server", "")!!
                net.language = sharedPreferences.getString("language", "en-US")!!
                val currentGroup = sharedPreferences.getString("currentGroup", "")!!.toInt()

                categoryDAO.eraseCategory(view.category.id, requireContext())
                val response = net.deleteCategory(currentGroup, view.category.id)
                activity!!.runOnUiThread{
                    Toast.makeText(context, response.body!!.string(), Toast.LENGTH_SHORT).show()
                }

                deleted = true
            }
        }

        if (!deleted) {
            Toast.makeText(requireContext(), getString(R.string.nothing_to_delete), Toast.LENGTH_SHORT).show()
        }
        updateContent()
    }

    fun updateSelected() {
        val layout = view?.findViewById<LinearLayout>(R.id.categoriesLayout)
        for (view: CategoryView in layout?.children!!.iterator() as Iterator<CategoryView>) {
            if (view.isCategorySelected) {
                val addCategoryIntent = Intent(context, AddCategoryActivity::class.java)
                val extras = Bundle()
                extras.putParcelable("category", view.category)
                addCategoryIntent.putExtras(extras)
                ContextCompat.startActivity(context!!, addCategoryIntent, extras)
            }
        }
    }

    fun updateContent() {
        val layout = view?.findViewById<LinearLayout>(R.id.categoriesLayout)
        layout?.removeAllViews()

        val categories = categoryDAO.getAllCategories()

        for (category in categories) {
            val categoryView = CategoryView(requireActivity(), requireContext(), category)
            layout?.addView(categoryView)
        }
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        updateContent()
    }
}