package org.foxarmy.barcodescannerforemployees.fragments

import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.core.content.ContextCompat
import androidx.core.view.children
import androidx.fragment.app.Fragment
import androidx.security.crypto.EncryptedSharedPreferences
import androidx.security.crypto.MasterKeys
import org.foxarmy.barcodescannerforemployees.Net
import org.foxarmy.barcodescannerforemployees.database.DBStorageController
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.activities.AddAbstractProductActivity
import org.foxarmy.barcodescannerforemployees.activities.FindBarcodelessAbstractProduct
import org.foxarmy.barcodescannerforemployees.database.AbstractProductDAO
import org.foxarmy.barcodescannerforemployees.databinding.FragmentStorageBinding
import org.foxarmy.barcodescannerforemployees.generateThumbnailForImage
import org.foxarmy.barcodescannerforemployees.views.AbstractProductView
import kotlin.concurrent.thread

class StorageFragment : Fragment() {

    private lateinit var binding: FragmentStorageBinding
    private var filterBy = ""
    private lateinit var filter: Array<String>

    private lateinit var sharedPreferences: SharedPreferences
    private lateinit var abstractProductDAO: AbstractProductDAO


    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        sharedPreferences = EncryptedSharedPreferences.create(
            "sensitive",
            MasterKeys.getOrCreate(MasterKeys.AES256_GCM_SPEC),
            requireContext(),
            EncryptedSharedPreferences.PrefKeyEncryptionScheme.AES256_SIV,
            EncryptedSharedPreferences.PrefValueEncryptionScheme.AES256_GCM
        )

        val dbHelper = DBStorageController(requireContext(), sharedPreferences.getString("currentGroup", "database")!!)
        abstractProductDAO = AbstractProductDAO(dbHelper)

        binding = FragmentStorageBinding.inflate(inflater)

        fillUpSortBySpinner()

        binding.spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onNothingSelected(parent: AdapterView<*>?) {
                updateContent()
            }

            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                updateContent()
            }
        }

        binding.dropFiltersButton.setOnClickListener {
            filterBy = ""
            updateContent()
        }

        filterBy = arguments?.getString("filterBy")!!
        filter = arguments?.getStringArray("filter")!!

        return binding.root
    }

    override fun onResume() {
        super.onResume()

        updateContent()
    }

    private fun fillUpSortBySpinner() {
        val sorts = resources.getStringArray(R.array.abstract_product_sort_types)
        val arrayAdapter = ArrayAdapter(requireContext(), androidx.appcompat.R.layout.support_simple_spinner_dropdown_item, sorts)
        arrayAdapter.setDropDownViewResource(androidx.appcompat.R.layout.support_simple_spinner_dropdown_item)
        binding.spinner.adapter = arrayAdapter
    }

    fun removeSelected() {
        thread {
            val grv = binding.contentGridLayout

            var deleted = false
            for (view: AbstractProductView in grv.children.iterator() as Iterator<AbstractProductView>) {
                activity!!.runOnUiThread {
                    view.findViewById<ImageView>(R.id.productPicture).setImageURI(null)
                }
                if (view.isProductSelected) {
                    val net = Net()
                    net.token = sharedPreferences.getString("token", "")!!
                    net.server = sharedPreferences.getString("server", "")!!
                    net.language = sharedPreferences.getString("language", "en-US")!!
                    val currentGroup = sharedPreferences.getString("currentGroup", "")!!.toInt()

                    abstractProductDAO.eraseAbstractProduct(view.abstractProduct.id, requireContext())
                    val response = net.deleteAbstractProduct(currentGroup, view.abstractProduct.id)
                    activity!!.runOnUiThread{
                        Toast.makeText(context, response.body!!.string(), Toast.LENGTH_SHORT).show()
                    }

                    deleted = true
                }
            }

            if (!deleted) {
                activity!!.runOnUiThread {
                    Toast.makeText(requireContext(), getString(R.string.nothing_to_delete), Toast.LENGTH_SHORT).show()
                }
            }
            updateContent()
        }
    }

    fun updateSelected() {
        val grv = binding.contentGridLayout

        for (view: AbstractProductView in grv.children.iterator() as Iterator<AbstractProductView>) {
            if (view.isProductSelected) {
                val addProductIntent = Intent(requireContext(), AddAbstractProductActivity::class.java)
                val extras = Bundle()
                extras.putParcelable("abstractProduct", view.abstractProduct)
                extras.putString("action", "update")
                addProductIntent.putExtras(extras)
                ContextCompat.startActivity(requireContext(), addProductIntent, extras)
            }
        }
    }

    fun updateContent() {
        thread {
            val grv = binding.contentGridLayout
            activity!!.runOnUiThread{
                grv.removeAllViews()
            }

            val abstractProducts = abstractProductDAO.getSortedListOfAbstractProducts(binding.spinner.selectedItemPosition, filterBy, filter)

            for (abstractProduct in abstractProducts) {
                generateThumbnailForImage(context!!, abstractProduct.imageHash)

                val abstractProductView = AbstractProductView(
                    requireActivity(),
                    requireContext(),
                    abstractProduct
                )

                if (filterBy == "barcodeless") {
                    abstractProductView.setOnClickListener {
                        (activity as FindBarcodelessAbstractProduct).selected(abstractProductView.abstractProduct)
                    }
                    abstractProductView.findViewById<TextView>(R.id.productNameView).setOnClickListener {
                        (activity as FindBarcodelessAbstractProduct).selected(abstractProductView.abstractProduct)
                    }
                }

                activity!!.runOnUiThread{
                    grv.addView(abstractProductView)
                }
            }
        }.join()
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        updateContent()
    }

    fun filterByCategory(id: Int) {
        filterBy = "category"
        filter = arrayOf("$id")
        updateContent()
    }

    companion object {

        fun newInstance(filterBy: String, filter: Array<String>):StorageFragment = StorageFragment().apply {
            val fragment = StorageFragment()
            val args = Bundle()
            args.putString("filterBy", filterBy)
            args.putStringArray("filter", filter)
            fragment.setArguments(args)

            return fragment
        }
    }
}