package org.foxarmy.barcodescannerforemployees.views

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import android.util.AttributeSet
import android.view.LayoutInflater
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import android.widget.Toast
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.core.content.ContextCompat
import androidx.core.content.ContextCompat.startActivity
import androidx.security.crypto.EncryptedSharedPreferences
import androidx.security.crypto.MasterKeys
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.activities.FullscreenActivity
import org.foxarmy.barcodescannerforemployees.database.CategoryDAO
import org.foxarmy.barcodescannerforemployees.database.DBStorageController
import org.foxarmy.barcodescannerforemployees.dataclasses.AbstractProduct
import org.foxarmy.barcodescannerforemployees.getActivity
import org.foxarmy.barcodescannerforemployees.getImageUri
import org.foxarmy.barcodescannerforemployees.getUnitNameById
import java.io.File

class AbstractProductView: LinearLayout {
    private var productLayout: ConstraintLayout? = null
    private var productPicture: ImageView? = null
    private var productNameField: TextView? = null
    private var netWeightField: TextView? = null
    private var categoryField: TextView? = null
    private var unitField: TextView? = null
    var abstractProduct: AbstractProduct = AbstractProduct()
    var isProductSelected = false
    private var activity: Activity

    private var categoryDAO: CategoryDAO
    private var sharedPreferences: SharedPreferences

    constructor(context: Context, a: AttributeSet) : super(context, a) {
        activity = getActivity(context)!!
        val inflater:LayoutInflater = activity.layoutInflater
        inflater.inflate(R.layout.abstract_product_view, this)

        sharedPreferences = EncryptedSharedPreferences.create(
            "sensitive",
            MasterKeys.getOrCreate(MasterKeys.AES256_GCM_SPEC),
            context,
            EncryptedSharedPreferences.PrefKeyEncryptionScheme.AES256_SIV,
            EncryptedSharedPreferences.PrefValueEncryptionScheme.AES256_GCM
        )

        val dbHelper = DBStorageController(context, sharedPreferences.getString("currentGroup", "database")!!)
        categoryDAO = CategoryDAO(dbHelper)
    }

    constructor(activity: Activity, context: Context, abstractProduct: AbstractProduct) : super(context) {
        this.abstractProduct = abstractProduct
        this.activity = activity
        val inflater:LayoutInflater = activity.layoutInflater
        inflater.inflate(R.layout.abstract_product_view, this)

        sharedPreferences = EncryptedSharedPreferences.create(
            "sensitive",
            MasterKeys.getOrCreate(MasterKeys.AES256_GCM_SPEC),
            context,
            EncryptedSharedPreferences.PrefKeyEncryptionScheme.AES256_SIV,
            EncryptedSharedPreferences.PrefValueEncryptionScheme.AES256_GCM
        )

        val dbHelper = DBStorageController(context, sharedPreferences.getString("currentGroup", "database")!!)
        categoryDAO = CategoryDAO(dbHelper)
        update()
    }

    fun init () {
        productLayout = findViewById(R.id.productLayout)
        productPicture = findViewById(R.id.productPicture)
        productNameField = findViewById(R.id.productNameView)
        netWeightField = findViewById(R.id.productNetWeightView)
        categoryField = findViewById(R.id.categoryView)
        unitField = findViewById(R.id.unitView)


        productPicture!!.setOnClickListener {
            val fullscreenIntent = Intent(activity, FullscreenActivity::class.java)
            val extras = Bundle()
            extras.putString("imagehash", abstractProduct.imageHash)
            fullscreenIntent.putExtras(extras)
            startActivity(context, fullscreenIntent, extras)
        }

        productLayout!!.setOnClickListener {

        }

        unitField?.text = getUnitNameById(context, abstractProduct.unit)

        productLayout!!.setOnLongClickListener {
            isProductSelected = !isProductSelected
            this.background = ContextCompat.getDrawable(context, if (isProductSelected) R.drawable.outline_selected else R.drawable.outline)
            true
        }

        productNameField!!.setOnClickListener {
            Toast.makeText(activity, productNameField!!.text, Toast.LENGTH_SHORT).show()
        }
    }

    fun update() {
        init()

        val thumbnailsDir = File(context.cacheDir, "thumbnails")
        thumbnailsDir.mkdirs()
        val imageUri = getImageUri(activity, File(thumbnailsDir, "${abstractProduct.imageHash}.webp"))
        productPicture!!.setImageURI(imageUri)
//        productPicture!!.rotation = 90f

        productNameField!!.text = abstractProduct.name
        netWeightField!!.text = abstractProduct.netWeight.toString()
        categoryField!!.text = categoryDAO.getCategoryNameById(abstractProduct.category)
    }
}