package org.foxarmy.barcodescannerforemployees.views

import android.app.Activity
import android.content.Context
import android.content.SharedPreferences
import android.view.LayoutInflater
import android.widget.LinearLayout
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.security.crypto.EncryptedSharedPreferences
import androidx.security.crypto.MasterKeys
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.activities.MainActivity
import org.foxarmy.barcodescannerforemployees.database.AbstractProductDAO
import org.foxarmy.barcodescannerforemployees.database.DBStorageController
import org.foxarmy.barcodescannerforemployees.dataclasses.Category

class CategoryView : LinearLayout {
    var category: Category
    val categoryName: TextView
    val amountOfProducts: TextView
    var isCategorySelected = false

    private var abstractProductDAO: AbstractProductDAO
    private var sharedPreferences: SharedPreferences

    constructor(activity: Activity, context: Context, category: Category) : super(context) {
        this.category = category

        sharedPreferences = EncryptedSharedPreferences.create(
            "sensitive",
            MasterKeys.getOrCreate(MasterKeys.AES256_GCM_SPEC),
            context,
            EncryptedSharedPreferences.PrefKeyEncryptionScheme.AES256_SIV,
            EncryptedSharedPreferences.PrefValueEncryptionScheme.AES256_GCM
        )

        val dbHelper = DBStorageController(context, sharedPreferences.getString("currentGroup", "database")!!)
        abstractProductDAO = AbstractProductDAO(dbHelper)

        val inflater: LayoutInflater = activity.layoutInflater
        inflater.inflate(R.layout.category_view, this)

        this.background = ContextCompat.getDrawable(context, if (isCategorySelected) R.drawable.outline_selected else R.drawable.outline)

        categoryName = findViewById(R.id.categoryNameTextView)
        amountOfProducts = findViewById(R.id.amountOfProducts)

        categoryName.text = category.name
        amountOfProducts.text = abstractProductDAO.getAmountOfAbstractProductsInCategory(category.id).toString()

        setOnLongClickListener {
            isCategorySelected = !isCategorySelected
            this.background = ContextCompat.getDrawable(context, if (isCategorySelected) R.drawable.outline_selected else R.drawable.outline)
            true
        }
        setOnClickListener {
            (activity as MainActivity).filterAbstractProductsByCategory(category.id)
        }
    }
}